@@ -1,5 +1,5 @@
 Clazz.declarePackage ("J.shape");
-Clazz.load (["J.shape.AtomShape", "java.util.Hashtable", "JU.P3"], "J.shape.Labels", ["javajs.awt.Font", "JU.AU", "$.BS", "$.Lst", "J.c.PAL", "JM.LabelToken", "$.Text", "JS.SV", "JU.BSUtil", "$.C", "JV.JC"], function () {
+Clazz.load (["J.shape.AtomShape", "java.util.Hashtable", "JU.P3"], "J.shape.Labels", ["javajs.awt.Font", "JU.AU", "$.BS", "$.Lst", "$.PT", "J.c.PAL", "JM.LabelToken", "$.Text", "JS.SV", "JU.BSUtil", "$.C", "JV.JC"], function () {
 c$ = Clazz.decorateAsClass (function () {
 this.strings = null;
 this.formats = null;
@@ -7,7 +7,6 @@ this.bgcolixes = null;
 this.fids = null;
 this.offsets = null;
 this.atomLabels = null;
-this.text = null;
 this.labelBoxes = null;
 this.bsFontSet = null;
 this.bsBgColixSet = null;
@@ -35,13 +34,14 @@ Clazz.prepareFields (c$, function () {
 this.atomLabels =  new java.util.Hashtable ();
 this.ptTemp =  new JU.P3 ();
 });
-Clazz.defineMethod (c$, "initShape", 
+Clazz.overrideMethod (c$, "initShape", 
 function () {
-Clazz.superCall (this, J.shape.Labels, "initShape", []);
 this.defaultFontId = this.zeroFontId = this.vwr.gdata.getFont3DFSS ("SansSerif", "Plain", 13).fid;
 this.defaultColix = 0;
 this.defaultBgcolix = 0;
-this.defaultOffset = J.shape.Labels.zeroOffset;
+this.defaultOffset = JV.JC.LABEL_DEFAULT_OFFSET;
+this.defaultAlignment = 4;
+this.defaultPointer = 0;
 this.defaultZPos = 0;
 this.translucentAllowed = false;
 });
@@ -54,9 +54,11 @@ return;
 }if ("color" === propertyName) {
 var pid = J.c.PAL.pidOf (value);
 var colix = JU.C.getColixO (value);
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setLabelColix (i, colix, pid);
+if (!this.setDefaults) {
+var n = this.checkColixLength (colix, bsSelected.length ());
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setLabelColix (i, colix, pid);
 
-if (this.setDefaults || !this.defaultsOnlyForNone) {
+}if (this.setDefaults || !this.defaultsOnlyForNone) {
 this.defaultColix = colix;
 this.defaultPaletteID = pid;
 }return;
@@ -64,44 +66,43 @@ this.defaultPaletteID = pid;
 if (this.strings == null) return;
 var val = (value).floatValue ();
 var scalePixelsPerMicron = (val == 0 ? 0 : 10000 / val);
-for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
-if (this.strings.length <= i) continue;
-this.text = this.getLabel (i);
-if (this.text == null) {
-this.text = JM.Text.newLabel (this.vwr, null, this.strings[i], 0, 0, 0, scalePixelsPerMicron, null);
-this.putLabel (i, this.text);
+var n = Math.min (this.ac, this.strings.length);
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) {
+var text = this.getLabel (i);
+if (text == null) {
+text = JM.Text.newLabel (this.vwr, null, this.strings[i], 0, 0, 0, scalePixelsPerMicron);
+this.putLabel (i, text);
 } else {
-this.text.setScalePixelsPerMicron (scalePixelsPerMicron);
+text.setScalePixelsPerMicron (scalePixelsPerMicron);
 }}
 return;
 }if ("label" === propertyName) {
 this.setScaling ();
 var tokens = null;
+var nbs = this.checkStringLength (bsSelected.length ());
+if (this.defaultColix != 0 || this.defaultPaletteID != 0) this.checkColixLength (this.defaultColix, bsSelected.length ());
+if (this.defaultBgcolix != 0) this.checkBgColixLength (this.defaultBgcolix, bsSelected.length ());
 if (Clazz.instanceOf (value, JU.Lst)) {
 var list = value;
 var n = list.size ();
-tokens = [null];
-for (var pt = 0, i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
+tokens =  Clazz.newArray (-1, [null]);
+for (var pt = 0, i = bsSelected.nextSetBit (0); i >= 0 && i < nbs; i = bsSelected.nextSetBit (i + 1)) {
 if (pt >= n) {
-this.setLabel (J.shape.Labels.nullToken, "", i);
-return;
+this.setLabel (J.shape.Labels.nullToken, "", i, true);
+continue;
 }tokens[0] = null;
-this.setLabel (tokens, JS.SV.sValue (list.get (pt++)), i);
+this.setLabel (tokens, JS.SV.sValue (list.get (pt++)), i, true);
 }
 } else {
 var strLabel = value;
-tokens = (strLabel == null || strLabel.length == 0 ? J.shape.Labels.nullToken : [null]);
-for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setLabel (tokens, strLabel, i);
+tokens = (strLabel == null || strLabel.length == 0 ? J.shape.Labels.nullToken :  Clazz.newArray (-1, [null]));
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setLabel (tokens, strLabel, i, true);
 
 }return;
-}if ("labels" === propertyName) {
+}if (propertyName.startsWith ("label:")) {
 this.setScaling ();
-var labels = value;
-for (var i = bsSelected.nextSetBit (0), pt = 0; i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
-var strLabel = labels.get (pt++);
-var tokens = (strLabel == null || strLabel.length == 0 ? J.shape.Labels.nullToken : [null]);
-this.setLabel (tokens, strLabel, i);
-}
+this.checkStringLength (this.ac);
+this.setLabel ( Clazz.newArray (-1, [null]), propertyName.substring (6), (value).intValue (), false);
 return;
 }if ("clearBoxes" === propertyName) {
 this.labelBoxes = null;
@@ -110,20 +111,16 @@ return;
 return;
 }if ("bgcolor" === propertyName) {
 this.isActive = true;
-if (this.bsBgColixSet == null) this.bsBgColixSet =  new JU.BS ();
+if (this.bsBgColixSet == null) this.bsBgColixSet = JU.BS.newN (this.ac);
 var bgcolix = JU.C.getColixO (value);
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setBgcolix (i, bgcolix);
+if (!this.setDefaults) {
+var n = this.checkBgColixLength (bgcolix, bsSelected.length ());
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setBgcolix (i, bgcolix);
 
-if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultBgcolix = bgcolix;
+}if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultBgcolix = bgcolix;
 return;
-}if (this.bsFontSet == null) this.bsFontSet =  new JU.BS ();
-if ("textLabels" === propertyName) {
-this.setScaling ();
-var labels = value;
-for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setTextLabel (i, labels.get (Integer.$valueOf (i)));
-
-return;
-}if ("fontsize" === propertyName) {
+}if (this.bsFontSet == null) this.bsFontSet = JU.BS.newN (this.ac);
+if ("fontsize" === propertyName) {
 var fontsize = (value).intValue ();
 if (fontsize < 0) {
 this.fids = null;
@@ -139,27 +136,23 @@ if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < thi
 
 if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultFontId = fid;
 return;
-}if ("offset" === propertyName || "offsetexact" === propertyName) {
+}if ("offset" === propertyName) {
 if (!(Clazz.instanceOf (value, Integer))) {
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setPymolOffset (i, value);
+if (!this.setDefaults) {
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setPymolOffset (i, value);
 
-return;
+}return;
 }var offset = (value).intValue ();
-var isExact = (propertyName === "offsetexact");
-if (offset == 0) offset = 32767;
- else if (offset == J.shape.Labels.zeroOffset) offset = 0;
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setOffsets (i, offset, isExact);
+if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setOffsets (i, offset);
 
 if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultOffset = offset;
 return;
 }if ("align" === propertyName) {
 var type = value;
-var alignment = 1;
-if (type.equalsIgnoreCase ("right")) alignment = 3;
- else if (type.equalsIgnoreCase ("center")) alignment = 2;
-for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setAlignment (i, alignment);
+var hAlignment = (type.equalsIgnoreCase ("right") ? 12 : type.equalsIgnoreCase ("center") ? 8 : 4);
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setHorizAlignment (i, hAlignment);
 
-if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultAlignment = alignment;
+if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultAlignment = hAlignment;
 return;
 }if ("pointer" === propertyName) {
 var pointer = (value).intValue ();
@@ -169,13 +162,13 @@ if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultPointer = pointer
 return;
 }if ("front" === propertyName) {
 var TF = (value).booleanValue ();
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setFront (i, TF);
+if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setZPos (i, 32, TF);
 
 if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultZPos = (TF ? 32 : 0);
 return;
 }if ("group" === propertyName) {
 var TF = (value).booleanValue ();
-if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setGroup (i, TF);
+if (!this.setDefaults) for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) this.setZPos (i, 16, TF);
 
 if (this.setDefaults || !this.defaultsOnlyForNone) this.defaultZPos = (TF ? 16 : 0);
 return;
@@ -188,14 +181,16 @@ var strLabelUNK = null;
 var tokensUNK = null;
 var strLabel;
 var tokens;
+var nstr = this.checkStringLength (bsSelected.length ());
+var bgcolix = this.defaultBgcolix;
+var nbg = this.checkBgColixLength (bgcolix, bsSelected.length ());
+var thisMad = (mode >= 0 ? 1 : -1);
 for (var i = bsSelected.nextSetBit (0); i >= 0 && i < this.ac; i = bsSelected.nextSetBit (i + 1)) {
 var atom = this.atoms[i];
-if (this.formats == null || i >= this.formats.length) this.formats = JU.AU.ensureLengthS (this.formats, i + 1);
-if (this.strings != null && this.strings.length > i && this.strings[i] != null) {
-this.mads[i] = (mode == 0 && this.mads[i] < 0 || mode == 1 ? 1 : -1);
+if (i < nstr && this.strings[i] != null) {
+this.mads[i] = (mode == 1 || mode == 0 && this.mads[i] < 0 ? 1 : -1);
 } else {
-if (this.bsSizeSet == null) this.bsSizeSet =  new JU.BS ();
-this.strings = JU.AU.ensureLengthS (this.strings, i + 1);
+this.mads[i] = thisMad;
 if (atom.getGroup3 (false).equals ("UNK")) {
 if (strLabelUNK == null) {
 strLabelUNK = this.vwr.getStandardLabelFormat (1);
@@ -211,14 +206,12 @@ tokens = tokensPDB;
 }this.strings[i] = JM.LabelToken.formatLabelAtomArray (this.vwr, atom, tokens, '\0', null, this.ptTemp);
 this.formats[i] = strLabel;
 this.bsSizeSet.set (i);
-if ((this.bsBgColixSet == null || !this.bsBgColixSet.get (i)) && this.defaultBgcolix != 0) this.setBgcolix (i, this.defaultBgcolix);
-this.mads[i] = (mode >= 0 ? 1 : -1);
-}this.setShapeVisibility (atom, this.strings != null && i < this.strings.length && this.strings[i] != null && this.mads[i] >= 0);
+if (i < nbg && !this.bsBgColixSet.get (i)) this.setBgcolix (i, this.defaultBgcolix);
+}atom.setShapeVisibility (this.vf, this.strings != null && i < this.strings.length && this.strings[i] != null && this.mads[i] >= 0);
 }
 return;
-}if (propertyName.startsWith ("label:")) {
-this.setScaling ();
-this.setLabel ( new Array (1), propertyName.substring (6), (value).intValue ());
+}if ("pymolLabels" === propertyName) {
+this.setPymolLabels (value, bsSelected);
 return;
 }if (propertyName === "deleteModelAtoms") {
 this.labelBoxes = null;
@@ -233,68 +226,92 @@ JU.BSUtil.deleteBits (this.bsFontSet, bsSelected);
 JU.BSUtil.deleteBits (this.bsBgColixSet, bsSelected);
 }this.setPropAS (propertyName, value, bsSelected);
 }, "~S,~O,JU.BS");
+Clazz.defineMethod (c$, "checkStringLength", 
+ function (n) {
+n = Math.min (this.ac, n);
+if (this.strings == null || n > this.strings.length) {
+this.formats = JU.AU.ensureLengthS (this.formats, n);
+this.strings = JU.AU.ensureLengthS (this.strings, n);
+if (this.bsSizeSet == null) this.bsSizeSet = JU.BS.newN (n);
+}return n;
+}, "~N");
+Clazz.defineMethod (c$, "checkBgColixLength", 
+ function (colix, n) {
+n = Math.min (this.ac, n);
+if (colix == 0) return (this.bgcolixes == null ? 0 : this.bgcolixes.length);
+if (this.bgcolixes == null || n > this.bgcolixes.length) this.bgcolixes = JU.AU.ensureLengthShort (this.bgcolixes, n);
+return n;
+}, "~N,~N");
+Clazz.defineMethod (c$, "setPymolLabels", 
+ function (labels, bsSelected) {
+this.setScaling ();
+var n = this.checkStringLength (this.ac);
+for (var i = bsSelected.nextSetBit (0); i >= 0 && i < n; i = bsSelected.nextSetBit (i + 1)) this.setPymolLabel (i, labels.get (Integer.$valueOf (i)), null);
+
+}, "java.util.Map,JU.BS");
 Clazz.defineMethod (c$, "setPymolOffset", 
  function (i, value) {
 var text = this.getLabel (i);
 if (text == null) {
-if (this.strings == null || this.strings.length <= i || this.strings[i] == null) return;
+if (this.strings == null || i >= this.strings.length || this.strings[i] == null) return;
 var fid = (this.bsFontSet != null && this.bsFontSet.get (i) ? this.fids[i] : -1);
 if (fid < 0) this.setFont (i, fid = this.defaultFontId);
-var font = javajs.awt.Font.getFont3D (fid);
-var colix = this.getColix2 (i, this.atoms[i], false);
-text = JM.Text.newLabel (this.vwr, font, this.strings[i], colix, this.getColix2 (i, this.atoms[i], true), 0, this.scalePixelsPerMicron, value);
-this.setTextLabel (i, text);
-} else {
-text.pymolOffset = value;
-}}, "~N,~A");
+text = JM.Text.newLabel (this.vwr, javajs.awt.Font.getFont3D (fid), this.strings[i], this.getColix2 (i, this.atoms[i], false), this.getColix2 (i, this.atoms[i], true), 0, this.scalePixelsPerMicron);
+this.setPymolLabel (i, text, this.formats[i]);
+}text.pymolOffset = value;
+}, "~N,~A");
 Clazz.defineMethod (c$, "setScaling", 
  function () {
 this.isActive = true;
-if (this.bsSizeSet == null) this.bsSizeSet =  new JU.BS ();
+if (this.bsSizeSet == null) this.bsSizeSet = JU.BS.newN (this.ac);
 this.isScaled = this.vwr.getBoolean (603979845);
 this.scalePixelsPerMicron = (this.isScaled ? this.vwr.getScalePixelsPerAngstrom (false) * 10000 : 0);
 });
-Clazz.defineMethod (c$, "setTextLabel", 
- function (i, t) {
+Clazz.defineMethod (c$, "setPymolLabel", 
+ function (i, t, format) {
 if (t == null) return;
-var label = t.getText ();
+var label = t.text;
 var atom = this.atoms[i];
-this.addString (atom, i, label, label);
-this.setShapeVisibility (atom, true);
+this.addString (atom, i, label, format == null ? JU.PT.rep (label, "%", "%%") : format);
+atom.setShapeVisibility (this.vf, true);
 if (t.colix >= 0) this.setLabelColix (i, t.colix, J.c.PAL.UNKNOWN.id);
 this.setFont (i, t.font.fid);
 this.putLabel (i, t);
-}, "~N,JM.Text");
+}, "~N,JM.Text,~S");
 Clazz.defineMethod (c$, "setLabel", 
- function (temp, strLabel, i) {
+ function (temp, strLabel, i, doAll) {
 var atom = this.atoms[i];
 var tokens = temp[0];
 if (tokens == null) tokens = temp[0] = JM.LabelToken.compile (this.vwr, strLabel, '\0', null);
 var label = (tokens == null ? null : JM.LabelToken.formatLabelAtomArray (this.vwr, atom, tokens, '\0', null, this.ptTemp));
-this.addString (atom, i, label, strLabel);
-this.text = this.getLabel (i);
-if (this.isScaled) {
-this.text = JM.Text.newLabel (this.vwr, null, label, 0, 0, 0, this.scalePixelsPerMicron, null);
-this.putLabel (i, this.text);
-} else if (this.text != null && label != null) {
-this.text.setText (label);
-}if (this.defaultOffset != J.shape.Labels.zeroOffset) this.setOffsets (i, this.defaultOffset, false);
-if (this.defaultAlignment != 1) this.setAlignment (i, this.defaultAlignment);
-if ((this.defaultZPos & 32) != 0) this.setFront (i, true);
- else if ((this.defaultZPos & 16) != 0) this.setGroup (i, true);
+var isNew = this.addString (atom, i, label, strLabel);
+doAll = new Boolean (doAll | (isNew || label == null)).valueOf ();
+var text = this.getLabel (i);
+if (this.isScaled && doAll) {
+text = JM.Text.newLabel (this.vwr, null, label, 0, 0, 0, this.scalePixelsPerMicron);
+this.putLabel (i, text);
+} else if (text != null && label != null) {
+text.setText (label);
+text.textUnformatted = strLabel;
+}if (!doAll) return;
+if (this.defaultOffset != JV.JC.LABEL_DEFAULT_OFFSET) this.setOffsets (i, this.defaultOffset);
+if (this.defaultAlignment != 4) this.setHorizAlignment (i, this.defaultAlignment);
+if ((this.defaultZPos & 32) != 0) this.setZPos (i, 32, true);
+ else if ((this.defaultZPos & 16) != 0) this.setZPos (i, 16, true);
 if (this.defaultPointer != 0) this.setPointer (i, this.defaultPointer);
 if (this.defaultColix != 0 || this.defaultPaletteID != 0) this.setLabelColix (i, this.defaultColix, this.defaultPaletteID);
 if (this.defaultBgcolix != 0) this.setBgcolix (i, this.defaultBgcolix);
 if (this.defaultFontId != this.zeroFontId) this.setFont (i, this.defaultFontId);
-}, "~A,~S,~N");
+}, "~A,~S,~N,~B");
 Clazz.defineMethod (c$, "addString", 
  function (atom, i, label, strLabel) {
-this.setShapeVisibility (atom, label != null);
-if (this.strings == null || i >= this.strings.length) this.strings = JU.AU.ensureLengthS (this.strings, i + 1);
-if (this.formats == null || i >= this.formats.length) this.formats = JU.AU.ensureLengthS (this.formats, i + 1);
+atom.setShapeVisibility (this.vf, label != null);
+var notNull = (strLabel != null);
+var isNew = (this.strings[i] == null);
 this.strings[i] = label;
-this.formats[i] = (strLabel != null && strLabel.indexOf ("%{") >= 0 ? label : strLabel);
-this.bsSizeSet.setBitTo (i, (strLabel != null));
+this.formats[i] = (notNull && strLabel.indexOf ("%{") >= 0 ? label : strLabel);
+this.bsSizeSet.setBitTo (i, notNull);
+return isNew;
 }, "JM.Atom,~N,~S,~S");
 Clazz.overrideMethod (c$, "getProperty", 
 function (property, index) {
@@ -305,8 +322,10 @@ return null;
 Clazz.defineMethod (c$, "putLabel", 
 function (i, text) {
 if (text == null) this.atomLabels.remove (Integer.$valueOf (i));
- else this.atomLabels.put (Integer.$valueOf (i), text);
-}, "~N,JM.Text");
+ else {
+this.atomLabels.put (Integer.$valueOf (i), text);
+text.textUnformatted = this.formats[i];
+}}, "~N,JM.Text");
 Clazz.defineMethod (c$, "getLabel", 
 function (i) {
 return this.atomLabels.get (Integer.$valueOf (i));
@@ -324,74 +343,65 @@ return this.labelBoxes.get (Integer.$valueOf (i));
 Clazz.defineMethod (c$, "setLabelColix", 
  function (i, colix, pid) {
 this.setColixAndPalette (colix, pid, i);
-if (this.colixes != null && ((this.text = this.getLabel (i)) != null)) this.text.colix = this.colixes[i];
+var text;
+if (this.colixes != null && ((text = this.getLabel (i)) != null)) text.colix = this.colixes[i];
 }, "~N,~N,~N");
 Clazz.defineMethod (c$, "setBgcolix", 
  function (i, bgcolix) {
-if (this.bgcolixes == null || i >= this.bgcolixes.length) {
-if (bgcolix == 0) return;
-this.bgcolixes = JU.AU.ensureLengthShort (this.bgcolixes, i + 1);
-}this.bgcolixes[i] = bgcolix;
+this.bgcolixes[i] = bgcolix;
 this.bsBgColixSet.setBitTo (i, bgcolix != 0);
-this.text = this.getLabel (i);
-if (this.text != null) this.text.bgcolix = bgcolix;
+var text = this.getLabel (i);
+if (text != null) text.bgcolix = bgcolix;
 }, "~N,~N");
 Clazz.defineMethod (c$, "setOffsets", 
- function (i, offset, isExact) {
+ function (i, offset) {
 if (this.offsets == null || i >= this.offsets.length) {
-if (offset == 0) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}this.offsets[i] = (this.offsets[i] & 255) | (offset << 8);
-if (isExact) this.offsets[i] |= 128;
-this.text = this.getLabel (i);
-if (this.text != null) this.text.setOffset (offset);
-}, "~N,~N,~B");
-Clazz.defineMethod (c$, "setAlignment", 
- function (i, alignment) {
+if (offset == JV.JC.LABEL_DEFAULT_OFFSET) return;
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
+}this.offsets[i] = (this.offsets[i] & 63) | offset;
+var text = this.getLabel (i);
+if (text != null) text.setOffset (offset);
+}, "~N,~N");
+Clazz.defineMethod (c$, "setHorizAlignment", 
+ function (i, hAlign) {
 if (this.offsets == null || i >= this.offsets.length) {
-if (alignment == 1) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}this.offsets[i] = (this.offsets[i] & -13) | (alignment << 2);
-this.text = this.getLabel (i);
-if (this.text != null) this.text.setAlignment (alignment);
+switch (hAlign) {
+case 0:
+case 4:
+return;
+}
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
+}if (hAlign == 0) hAlign = 4;
+this.offsets[i] = JV.JC.setHorizAlignment (this.offsets[i], hAlign);
+var text = this.getLabel (i);
+if (text != null) text.setAlignment (hAlign);
 }, "~N,~N");
-c$.getAlignment = Clazz.defineMethod (c$, "getAlignment", 
-function (offsetFull) {
-return (offsetFull & 12) >> 2;
-}, "~N");
 Clazz.defineMethod (c$, "setPointer", 
  function (i, pointer) {
 if (this.offsets == null || i >= this.offsets.length) {
 if (pointer == 0) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}this.offsets[i] = (this.offsets[i] & -4) + pointer;
-this.text = this.getLabel (i);
-if (this.text != null) this.text.pointer = pointer;
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
+}this.offsets[i] = JV.JC.setPointer (this.offsets[i], pointer);
+var text = this.getLabel (i);
+if (text != null) text.pointer = pointer;
 }, "~N,~N");
-Clazz.defineMethod (c$, "setFront", 
- function (i, TF) {
+Clazz.defineMethod (c$, "setZPos", 
+ function (i, flag, TF) {
 if (this.offsets == null || i >= this.offsets.length) {
 if (!TF) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}this.offsets[i] = (this.offsets[i] & -49) + (TF ? 32 : 0);
-}, "~N,~B");
-Clazz.defineMethod (c$, "setGroup", 
- function (i, TF) {
-if (this.offsets == null || i >= this.offsets.length) {
-if (!TF) return;
-this.offsets = JU.AU.ensureLengthI (this.offsets, i + 1);
-}this.offsets[i] = (this.offsets[i] & -49) + (TF ? 16 : 0);
-}, "~N,~B");
+this.offsets = JU.AU.ensureLengthI (this.offsets, this.ac);
+}this.offsets[i] = JV.JC.setZPosition (this.offsets[i], TF ? flag : 0);
+}, "~N,~N,~B");
 Clazz.defineMethod (c$, "setFont", 
  function (i, fid) {
 if (this.fids == null || i >= this.fids.length) {
 if (fid == this.zeroFontId) return;
-this.fids = JU.AU.ensureLengthByte (this.fids, i + 1);
+this.fids = JU.AU.ensureLengthByte (this.fids, this.ac);
 }this.fids[i] = fid;
 this.bsFontSet.set (i);
-this.text = this.getLabel (i);
-if (this.text != null) {
-this.text.setFontFromFid (fid);
+var text = this.getLabel (i);
+if (text != null) {
+text.setFontFromFid (fid);
 }}, "~N,~N");
 Clazz.overrideMethod (c$, "setAtomClickability", 
 function () {
@@ -401,11 +411,6 @@ var label = this.strings[i];
 if (label != null && this.ms.at.length > i && !this.ms.isAtomHidden (i)) this.ms.at[i].setClickable (this.vf);
 }
 });
-Clazz.overrideMethod (c$, "getShapeState", 
-function () {
-if (!this.isActive || this.bsSizeSet == null) return "";
-return this.vwr.getShapeState (this);
-});
 Clazz.overrideMethod (c$, "checkObjectDragged", 
 function (prevX, prevY, x, y, dragAction, bsVisible) {
 if (this.vwr.getPickingMode () != 2 || this.labelBoxes == null) return false;
@@ -413,9 +418,10 @@ if (prevX == -2147483648) {
 var iAtom = this.findNearestLabel (x, y);
 if (iAtom >= 0) {
 this.pickedAtom = iAtom;
+this.vwr.acm.setDragAtomIndex (iAtom);
 this.pickedX = x;
 this.pickedY = y;
-this.pickedOffset = (this.offsets == null || this.pickedAtom >= this.offsets.length ? 0 : this.offsets[this.pickedAtom]) >> 8;
+this.pickedOffset = (this.offsets == null || this.pickedAtom >= this.offsets.length ? JV.JC.LABEL_DEFAULT_OFFSET : this.offsets[this.pickedAtom]);
 return true;
 }return false;
 }if (prevX == 2147483647) {
@@ -431,11 +437,12 @@ if (this.labelBoxes == null) return -1;
 var dmin = 3.4028235E38;
 var imin = -1;
 var zmin = 3.4028235E38;
+var afactor = (this.vwr.antialiased ? 2 : 1);
 for (var entry, $entry = this.labelBoxes.entrySet ().iterator (); $entry.hasNext () && ((entry = $entry.next ()) || true);) {
 if (!this.atoms[entry.getKey ().intValue ()].isVisible (this.vf | 9)) continue;
 var boxXY = entry.getValue ();
-var dx = x - boxXY[0];
-var dy = y - boxXY[1];
+var dx = (x - boxXY[0]) * afactor;
+var dy = (y - boxXY[1]) * afactor;
 if (dx <= 0 || dy <= 0 || dx >= boxXY[2] || dy >= boxXY[3] || boxXY[4] > zmin) continue;
 zmin = boxXY[4];
 var d = Math.min (Math.abs (dx - boxXY[2] / 2), Math.abs (dy - boxXY[3] / 2));
@@ -448,13 +455,11 @@ return imin;
 Clazz.defineMethod (c$, "move2D", 
  function (pickedAtom, x, y) {
 var xOffset = JV.JC.getXOffset (this.pickedOffset);
-var yOffset = -JV.JC.getYOffset (this.pickedOffset);
+var yOffset = JV.JC.getYOffset (this.pickedOffset);
 xOffset += x - this.pickedX;
-yOffset += this.pickedY - y;
-var offset = JV.JC.getOffset (xOffset, yOffset);
-if (offset == 0) offset = 32767;
- else if (offset == J.shape.Labels.zeroOffset) offset = 0;
-this.setOffsets (pickedAtom, offset, true);
+yOffset -= y - this.pickedY;
+var offset = JV.JC.getOffset (xOffset, yOffset, true);
+this.setOffsets (pickedAtom, offset);
 }, "~N,~N,~N");
 Clazz.defineMethod (c$, "getColix2", 
 function (i, atom, isBg) {
@@ -467,7 +472,5 @@ colix = JU.C.getColixInherited (colix, atom.colixAtom);
 if (JU.C.isColixTranslucent (colix)) colix = JU.C.getColixTranslucent3 (colix, false, 0);
 }return colix;
 }, "~N,JM.Atom,~B");
-Clazz.defineStatics (c$,
-"zeroOffset", 1028);
-c$.nullToken = c$.prototype.nullToken = [null];
+c$.nullToken = c$.prototype.nullToken =  Clazz.newArray (-1, [null]);
 });
